Public Delegate Function MyFxDelegate(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double) As Double
Public Delegate Function SayFxDelegate() As String

Public Class CRandomSearch2
  Dim m_MyFx As MyFxDelegate

  Protected Function MyFxEx(ByVal N As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
                            ByRef fDeltaX() As Double, ByVal fLambda As Double) As Double
    Dim I As Integer
    Dim fX2() As Double

    ReDim fX2(N)

    For I = 0 To N - 1
      fX2(I) = fX(I) + fLambda * fDeltaX(I)
    Next I

    Return m_MyFx(N, fX2, fParam)
  End Function

  Protected Function LinSearch_DirectSearch(ByVal N As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
            ByRef fLambda As Double, ByRef fDeltaX() As Double, ByVal fInittep As Double, ByVal fMinStep As Double) As Boolean
    Dim F1 As Double, F2 As Double

    F1 = MyFxEx(N, fX, fParam, fDeltaX, fLambda)

    Do
      F2 = MyFxEx(N, fX, fParam, fDeltaX, fLambda + fInittep)
      If F2 < F1 Then
        F1 = F2
        fLambda = fLambda + fInittep
      Else
        F2 = MyFxEx(N, fX, fParam, fDeltaX, fLambda - fInittep)
        If F2 < F1 Then
          F1 = F2
          fLambda = fLambda - fInittep
        Else
          ' reduce search step size
          fInittep = fInittep / 10
        End If
      End If
    Loop Until fInittep < fMinStep

    LinSearch_DirectSearch = True

  End Function

  Public Function CalcOptim(ByVal nNumVars As Integer, ByRef fX() As Double, ByRef fParam() As Double, _
                            ByRef fXLo() As Double, ByRef fXHi() As Double, _
                            ByVal nMaxIter As Integer, ByVal EpsFx As Double, _
                            ByRef nIter As Integer, ByVal MyFx As MyFxDelegate) As Double

    Dim fDeltaX() As Double
    Dim fInitStep As Double = 0.1, fMinStep As Double = 0.00001

    Dim F, fBestF, fBestX(), fLastBestF, fLambda As Double
    Dim I As Integer

    ReDim fDeltaX(nNumVars)
    m_MyFx = MyFx
    ReDim fBestX(nNumVars)

    For I = 0 To nNumVars - 1
      fBestX(I) = fX(I)
    Next
    ' calculate and display function value at initial point
    fBestF = MyFx(nNumVars, fBestX, fParam)
    If fBestF > 0 Then
      fLastBestF = fBestF + 100
    Else
      fLastBestF = 100 - fBestF
    End If

    nIter = 0
    Do
      nIter += 1
      If nIter > nMaxIter Then Exit Do
      Randomize(Timer)
      For I = 0 To nNumVars - 1
        fX(I) = fXLo(I) + Rnd(1) * (fXHi(I) - fXLo(I))
        fDeltaX(I) = fX(I) - fBestX(I)
      Next
      fLambda = 0.1
      LinSearch_DirectSearch(nNumVars, fBestX, fParam, fLambda, fDeltaX, fInitStep, fMinStep)
      For I = 0 To nNumVars - 1
        fX(I) = fBestX(I) + fLambda * fDeltaX(I)
      Next
      F = MyFx(nNumVars, fX, fParam)
      If F < fBestF Then
        For I = 0 To nNumVars - 1
          fBestX(I) = fX(I)
        Next
        fBestF = F

        ' test function value convergence
        If Math.Abs(fBestF - fLastBestF) < EpsFx Then Exit Do
        fLastBestF = fBestF
      End If
    Loop

    Return fBestF

  End Function
End Class
